/* global media_type, Port */
"use strict";

// should not run on image pages
if (!media_type() && document instanceof window.HTMLDocument
	&& (!browser.safari || location.protocol !== "safari-extension:")) {
	Port.listen(function(cfg) {
		Port.listen(null);

		cfg = JSON.parse((cfg && (cfg.data || cfg).prefs)).opener;

		if (!cfg) {
			return;
		}

		var mdownstart, md_x, md_y,
			checkBG = function(imgs) {
				// ("...\"") - Gecko
				// (...) or ('...)') - WebKit
				// ("...&quot;") - Presto
				imgs = (imgs.backgroundImage + imgs.content + imgs.listStyleImage).match(/\burl\(([^'"\)][^\)]*|"[^"\\]+(?:\\.[^"\\]*)*|'[^'\\]+(?:\\.[^'\\]*)*)(?=['"]?\))/g);

				if (Array.isArray(imgs)) {
					for (var i = 0; i < imgs.length; ++i) {
						imgs[i] = imgs[i].slice(/'|"/.test(imgs[i][4]) ? 5 : 4);
					}
				}

				return imgs;
			},

			checkIMG = function(node) {
				var nname = node.nodeName.toUpperCase();

				if (nname === "IMG" || node.type === "image" || nname === "EMBED") {
					return node.src;
				}
				else if (nname === "CANVAS") {
					return node.toDataURL();
				}
				else if (nname === "OBJECT" && node.data) {
					return node.data;
				}
				else if (nname === "AREA") {
					var img = document.querySelector('img[usemap="#' + node.parentNode.name + '"]');
					return img.src;
				}
				else if (nname === "VIDEO") {
					nname = document.createElement("canvas");
					nname.width = node.clientWidth;
					nname.height = node.clientHeight;
					nname.getContext("2d").drawImage(node, 0, 0, nname.width, nname.height);
					return nname.toDataURL("image/jpeg");
				}
				else if (window.SVGDocument && node.document instanceof window.SVGDocument
					|| /^\[object SVG/.test(node.toString())) {
					if (node.ownerDocument && !/^\[object SVG/.test(node.ownerDocument.toString())) {
						node = (new window.XMLSerializer).serializeToString(
							node.ownerSVGElement === null ? node : node.ownerSVGElement
						);

						if (typeof node === "string") {
							return "data:image/svg+xml," + encodeURIComponent(node);
						}
					}
				}
				else if (node.poster) {
					return node.poster;
				}

				return null;
			};

		window.addEventListener("mousedown", function(e) {
			if (e.button === 2) {
				mdownstart = Date.now(); // e.timeStamp
				// context menu should work where the mousedown event happened (Chrome...)
				md_x = e.clientX;
				md_y = e.clientY;
			}
		}, true);

		window.addEventListener("contextmenu", function(e) {
			if (!mdownstart || e.button !== 2
				|| md_x != e.clientX || md_y != e.clientY) {
				return;
			}

			var elapsed = Date.now() - mdownstart >= 300;

			if (!(e.ctrlKey && e.altKey && !e.shiftKey && !elapsed && cfg > 1
					|| elapsed && (cfg === 1 || cfg === 3) && !e.ctrlKey && !e.shiftKey && !e.altKey)) {
				return;
			}

			var url, urls = [], r, el = 30, rgxHTMLBODY = /HTML|BODY/i;
			// not exactly what we want
			var XPath = document.evaluate(
				". | ./ancestor::*[position()<" + el
				+ "] | ./preceding::*[position()<" + el
				+ "][not(head) and not(ancestor::head)] | ./descendant::*[position()<"
				+ el + "] | ./following::*[position()<" + el + "]",
				e.target, null, 4, null
			);

			while (el = XPath.iterateNext()) {
				r = el.getBoundingClientRect();

				if (e.clientX >= r.left && e.clientX <= r.left + r.width
					&& e.clientY >= r.top && e.clientY <= r.top + r.height) {
					if (rgxHTMLBODY.test(el.nodeName) && !rgxHTMLBODY.test(e.target.nodeName)) {
						continue;
					}

					if (url = checkIMG(el)) {
						urls.push(url);
					}

					if (url = checkBG(window.getComputedStyle(el))) {
						urls = urls.concat(url);
					}
				}
			}

			if (urls && urls.length) {
				e.stopPropagation();
				e.preventDefault();

				var i, filter = {};

				for (i = 0; i < urls.length; ++i) {
					filter[urls[i]] = true;
				}

				Port.send({
					"cmd": "open",
					"url": Object.keys(filter).reverse()
				});
			}
		}, true);
	});

	Port.send({"cmd": "hello"});
}